@@ -244,6 +244,9 @@ fn main() {
                         if val.len() > 1 {
                             txt.pop();
                             out.set_value(txt.as_str());
+                        } else {
+                            txt.pop();
+                            out.set_value("0");
                         }
                     }
                     Ops::CE => {
