@@ -21,6 +21,9 @@ use std::{
 /// Alias Widget ptr
 pub type WidgetPtr = *mut fltk_sys::widget::Fl_Widget;
 
+/// Alias Window ptr
+pub type WindowPtr = *mut fltk_sys::window::Fl_Window;
+
 lazy_static! {
     /// The currently chosen font
     static ref CURRENT_FONT: Mutex<i32> = Mutex::new(0);
@@ -182,20 +185,22 @@ impl App {
         self
     }
 
-    /// Loads a font from a path.
-    /// On success, returns a String with the ttf Font Family name. The font's index is always 16.
-    /// As such only one font can be loaded at a time.
-    /// The font name can be used with `Font::by_name`, and index with `Font::by_index`.
-    /// # Examples
-    /// ```rust,no_run
-    /// use fltk::{prelude::*, *};
-    /// let app = app::App::default();
-    /// let font = app.load_font("font.ttf").unwrap();
-    /// let mut frame = frame::Frame::new(0, 0, 400, 100, "Hello");
-    /// frame.set_label_font(enums::Font::by_name(&font));
-    /// ```
-    /// # Errors
-    /// Returns `ResourceNotFound` if the Font file was not found
+    /**
+        Loads a font from a path.
+        On success, returns a String with the ttf Font Family name. The font's index is always 16.
+        As such only one font can be loaded at a time.
+        The font name can be used with `Font::by_name`, and index with `Font::by_index`.
+        # Examples
+        ```rust,no_run
+        use fltk::{prelude::*, *};
+        let app = app::App::default();
+        let font = app.load_font("font.ttf").unwrap();
+        let mut frame = frame::Frame::new(0, 0, 400, 100, "Hello");
+        frame.set_label_font(enums::Font::by_name(&font));
+        ```
+        # Errors
+        Returns `ResourceNotFound` if the Font file was not found
+    */
     pub fn load_font<P: AsRef<path::Path>>(self, path: P) -> Result<String, FltkError> {
         Self::load_font_(path.as_ref())
     }
@@ -265,17 +270,21 @@ pub fn set_grab<W: WindowExt>(win: Option<W>) {
 /// Returns the latest captured event
 pub fn event() -> Event {
     unsafe {
-        let x = fl::Fl_event();
-        let x: Event = mem::transmute(x);
-        x
+        mem::transmute(fl::Fl_event())
     }
 }
 
 /// Returns the presed key
 pub fn event_key() -> Key {
     unsafe {
-        let x = fl::Fl_event_key();
-        mem::transmute(x)
+        mem::transmute(fl::Fl_event_key())
+    }
+}
+
+/// Returns the original key
+pub fn event_original_key() -> Key {
+    unsafe {
+        mem::transmute(fl::Fl_event_original_key())
     }
 }
 
@@ -415,77 +424,121 @@ pub fn event_state() -> Shortcut {
 
 /// Returns a pair of the width and height of the screen
 pub fn screen_size() -> (f64, f64) {
-    unsafe {
-        (
-            (fl::Fl_screen_w() as f64),
-            (fl::Fl_screen_h() as f64),
-        )
-    }
+    unsafe { ((fl::Fl_screen_w() as f64), (fl::Fl_screen_h() as f64)) }
 }
 
 /// Returns a pair of the x & y coords of the screen
 pub fn screen_coords() -> (i32, i32) {
-    unsafe {
-        (fl::Fl_screen_x(), fl::Fl_screen_y())
+    unsafe { (fl::Fl_screen_x(), fl::Fl_screen_y()) }
+}
+
+/// Types of Clipboard contents
+#[derive(Debug, Clone, Copy)]
+pub enum ClipboardContent {
+    /// Textual content
+    Text,
+    /// Image content
+    Image,
+}
+
+/// Check the contents of the clipboard
+pub fn clipboard_contains(content: ClipboardContent) -> bool {
+    use ClipboardContent::*;
+    let txt = match content {
+        Text => "text/plain",
+        Image => "image",
+    };
+    let txt = CString::new(txt).unwrap();
+    unsafe { fl::Fl_clipboard_contains(txt.as_ptr()) != 0 }
+}
+
+/// Pastes content from the clipboard
+pub fn paste<T>(widget: &T)
+where
+    T: WidgetExt,
+{
+    assert!(!widget.was_deleted());
+    if clipboard_contains(ClipboardContent::Text) {
+        paste_text(widget)
+    } else if clipboard_contains(ClipboardContent::Image) {
+        paste_image(widget)
+    } else {
+        // Do nothing!
     }
 }
 
-/// Used for widgets implementing the `InputExt`, pastes content from the clipboard
-pub fn paste<T>(widget: &T)
+/// Pastes textual content from the clipboard
+pub fn paste_text<T>(widget: &T)
 where
-    T: WidgetBase + InputExt,
+    T: WidgetExt,
 {
     assert!(!widget.was_deleted());
     unsafe {
-        fl::Fl_paste(widget.as_widget_ptr() as *mut fltk_sys::fl::Fl_Widget, 1);
+        fl::Fl_paste_text(widget.as_widget_ptr() as *mut fltk_sys::fl::Fl_Widget, 1);
+    }
+}
+
+/// Pastes image content from the clipboard
+pub fn paste_image<T>(widget: &T)
+where
+    T: WidgetExt,
+{
+    assert!(!widget.was_deleted());
+    unsafe {
+        fl::Fl_paste_image(widget.as_widget_ptr() as *mut fltk_sys::fl::Fl_Widget, 1);
     }
 }
 
 /// Sets the callback of a widget
 pub fn set_callback<F, W>(widget: &mut W, cb: F)
 where
-    F: FnMut(),
+    F: FnMut(&mut dyn WidgetExt),
     W: WidgetExt,
 {
     assert!(!widget.was_deleted());
     unsafe {
-        unsafe extern "C" fn shim(_wid: *mut fltk_sys::widget::Fl_Widget, data: *mut raw::c_void) {
-            let a: *mut Box<dyn FnMut()> = data as *mut Box<dyn FnMut()>;
-            let f: &mut (dyn FnMut()) = &mut **a;
-            let _ = panic::catch_unwind(panic::AssertUnwindSafe(|| f()));
+        unsafe extern "C" fn shim(wid: *mut fltk_sys::widget::Fl_Widget, data: *mut raw::c_void) {
+            let a: *mut Box<dyn FnMut(&mut dyn WidgetExt)> =
+                data as *mut Box<dyn FnMut(&mut dyn WidgetExt)>;
+            let f: &mut (dyn FnMut(&mut dyn WidgetExt)) = &mut **a;
+            let mut wid = crate::widget::Widget::from_widget_ptr(wid);
+            let _ = panic::catch_unwind(panic::AssertUnwindSafe(|| f(&mut wid)));
         }
         let _old_data = widget.user_data();
-        let a: *mut Box<dyn FnMut()> = Box::into_raw(Box::new(Box::new(cb)));
+        let a: *mut Box<dyn FnMut(&mut dyn WidgetExt)> = Box::into_raw(Box::new(Box::new(cb)));
         let data: *mut raw::c_void = a as *mut raw::c_void;
         let callback: fltk_sys::widget::Fl_Callback = Some(shim);
         fltk_sys::widget::Fl_Widget_set_callback(widget.as_widget_ptr(), callback, data);
     }
 }
 
-/// Set a widget callback using a C style API
-/// ```rust,no_run
-/// use fltk::{prelude::*, *};
-/// use std::os::raw::*;
-/// // data can be anything, even a different widget
-/// fn cb(w: app::WidgetPtr, data: *mut c_void) {
-///     // To access the button
-///     let mut btn = unsafe { button::Button::from_widget_ptr(w) }; // Gets a Widget
-///     btn.set_label("Works!");
-///     // To access the frame
-///     let mut frm = unsafe { widget::Widget::from_widget_ptr(data as app::WidgetPtr) };
-///     frm.set_label("Works!");
-/// }
-/// let mut but = button::Button::default();
-/// let mut frame = frame::Frame::default();
-/// unsafe {
-///     // If no data needs to be passed, you can pass 0 as *mut _
-///     app::set_raw_callback(&mut but, frame.as_widget_ptr() as *mut _, Some(cb));
-///     // Using a closure also works
-///     app::set_raw_callback(&mut but, frame.as_widget_ptr() as *mut _, Some(|_ , _| { println!("Also works!")}));
-/// }
-/// ```
-/// # Safety
-/// The function involves dereferencing externally provided raw pointers
+#[allow(clippy::missing_safety_doc)]
+/**
+    Set a widget callback using a C style API
+    ```rust,no_run
+    use fltk::{prelude::*, *};
+    use std::os::raw::*;
+    // data can be anything, even a different widget
+    fn cb(w: app::WidgetPtr, data: *mut c_void) {
+        // To access the button
+        let mut btn = unsafe { button::Button::from_widget_ptr(w) }; // Gets a Widget
+        btn.set_label("Works!");
+        // To access the frame
+        let mut frm = unsafe { widget::Widget::from_widget_ptr(data as app::WidgetPtr) };
+        frm.set_label("Works!");
+    }
+    let mut but = button::Button::default();
+    let mut frame = frame::Frame::default();
+    unsafe {
+        // If no data needs to be passed, you can pass 0 as *mut _
+        app::set_raw_callback(&mut but, frame.as_widget_ptr() as *mut _, Some(cb));
+        // Using a closure also works
+        app::set_raw_callback(&mut but, frame.as_widget_ptr() as *mut _, Some(|_ , _| { println!("Also works!")}));
+    }
+    ```
+    # Safety
+    The function involves dereferencing externally provided raw pointers
+*/
 pub unsafe fn set_raw_callback<W>(
     widget: &mut W,
     data: *mut raw::c_void,
@@ -510,6 +563,22 @@ pub fn set_visible_focus(flag: bool) {
 
 /// Set the app's default frame type
 pub fn set_frame_type(new_frame: FrameType) {
+    unsafe {
+        let new_frame = new_frame as i32;
+        let mut curr = CURRENT_FRAME.lock().unwrap();
+        fl::Fl_set_box_type(*curr, new_frame);
+        *curr = new_frame;
+    }
+}
+
+/// Get the app's frame type
+pub fn frame_type() -> FrameType {
+    let curr = CURRENT_FRAME.lock().unwrap();
+    FrameType::by_index(*curr as _)
+}
+
+/// Swap the default frame type with a new frame type
+pub fn swap_frame_type(new_frame: FrameType) {
     unsafe {
         let new_frame = new_frame as i32;
         let mut curr = CURRENT_FRAME.lock().unwrap();
@@ -690,13 +759,16 @@ pub unsafe fn awake_msg<T>(msg: T) {
     fl::Fl_awake_msg(Box::into_raw(Box::from(msg)) as *mut raw::c_void);
 }
 
-/// Receives a custom message
-/// ```rust,no_run
-/// use fltk::{prelude::*, *};
-/// if let Some(msg) = unsafe { app::thread_msg::<i32>() } { /* do something */ }
-/// ```
-/// # Safety
-/// The type must correspond to the received message
+#[allow(clippy::missing_safety_doc)]
+/**
+    Receives a custom message
+    ```rust,no_run
+    use fltk::{prelude::*, *};
+    if let Some(msg) = unsafe { app::thread_msg::<i32>() } { /* do something */ }
+    ```
+    # Safety
+    The type must correspond to the received message
+*/
 pub unsafe fn thread_msg<T>() -> Option<T> {
     let msg = fl::Fl_thread_msg();
     if msg.is_null() {
@@ -819,22 +891,24 @@ pub fn quit() {
     }
 }
 
-/// Adds a one-shot timeout callback. The timeout duration `tm` is indicated in seconds
-/// Example:
-/// ```rust,no_run
-/// use fltk::{prelude::*, *};
-/// fn callback() {
-///     println!("TICK");
-///     app::repeat_timeout(1.0, callback);
-/// }
-/// fn main() {
-///     let app = app::App::default();
-///     let mut wind = window::Window::new(100, 100, 400, 300, "");
-///     wind.show();
-///     app::add_timeout(1.0, callback);
-///     app.run().unwrap();
-/// }
-/// ```
+/**
+    Adds a one-shot timeout callback. The timeout duration `tm` is indicated in seconds
+    Example:
+    ```rust,no_run
+    use fltk::{prelude::*, *};
+    fn callback() {
+        println!("TICK");
+        app::repeat_timeout(1.0, callback);
+    }
+    fn main() {
+        let app = app::App::default();
+        let mut wind = window::Window::new(100, 100, 400, 300, "");
+        wind.show();
+        app::add_timeout(1.0, callback);
+        app.run().unwrap();
+    }
+    ```
+*/
 pub fn add_timeout<F: FnMut() + 'static>(tm: f64, cb: F) {
     unsafe {
         unsafe extern "C" fn shim(data: *mut raw::c_void) {
@@ -849,24 +923,26 @@ pub fn add_timeout<F: FnMut() + 'static>(tm: f64, cb: F) {
     }
 }
 
-/// Repeats a timeout callback from the expiration of the previous timeout.
-/// You may only call this method inside a timeout callback.
-/// The timeout duration `tm` is indicated in seconds
-/// Example:
-/// ```rust,no_run
-/// use fltk::{prelude::*, *};
-/// fn callback() {
-///     println!("TICK");
-///     app::repeat_timeout(1.0, callback);
-/// }
-/// fn main() {
-///     let app = app::App::default();
-///     let mut wind = window::Window::new(100, 100, 400, 300, "");
-///     wind.show();
-///     app::add_timeout(1.0, callback);
-///     app.run().unwrap();
-/// }
-/// ```
+/**
+    Repeats a timeout callback from the expiration of the previous timeout.
+    You may only call this method inside a timeout callback.
+    The timeout duration `tm` is indicated in seconds
+    Example:
+    ```rust,no_run
+    use fltk::{prelude::*, *};
+    fn callback() {
+        println!("TICK");
+        app::repeat_timeout(1.0, callback);
+    }
+    fn main() {
+        let app = app::App::default();
+        let mut wind = window::Window::new(100, 100, 400, 300, "");
+        wind.show();
+        app::add_timeout(1.0, callback);
+        app.run().unwrap();
+    }
+    ```
+*/
 pub fn repeat_timeout<F: FnMut() + 'static>(tm: f64, cb: F) {
     unsafe {
         unsafe extern "C" fn shim(data: *mut raw::c_void) {
@@ -936,13 +1012,15 @@ pub fn event_inside(x: i32, y: i32, w: i32, h: i32) -> bool {
     unsafe { fl::Fl_event_inside(x, y, w, h) != 0 }
 }
 
-/// Gets the widget that is below the mouse cursor.
-/// This returns an Option<impl WidgetExt> which can be specified in the function call
-/// ```rust,no_run
-/// use fltk::app;
-/// use fltk::widget;
-/// let w = app::belowmouse::<widget::Widget>(); // or by specifying a more concrete type
-/// ```
+/**
+    Gets the widget that is below the mouse cursor.
+    This returns an Option<impl WidgetExt> which can be specified in the function call
+    ```rust,no_run
+    use fltk::app;
+    use fltk::widget;
+    let w = app::belowmouse::<widget::Widget>(); // or by specifying a more concrete type
+    ```
+*/
 pub fn belowmouse<Wid: WidgetExt>() -> Option<impl WidgetExt> {
     unsafe {
         let x = fl::Fl_belowmouse() as *mut fltk_sys::fl::Fl_Widget;
@@ -1160,6 +1238,7 @@ fn load_font(path: &str) -> Result<String, FltkError> {
             } else {
                 f[16] = name.clone();
             }
+            fl::Fl_set_font2(16, ptr);
             Ok(name)
         }
     }
@@ -1225,31 +1304,33 @@ pub fn get_system_colors() {
     unsafe { fl::Fl_get_system_colors() }
 }
 
-/// Send a signal to a window.
-/// Integral values from 0 to 30 are reserved.
-/// Returns Ok(true) if the event was handled.
-/// Returns Ok(false) if the event was not handled.
-/// Returns Err on error or in use of one of the reserved values.
-/// ```rust,no_run
-/// use fltk::{prelude::*, *};
-/// const CHANGE_FRAME: i32 = 100;
-/// let mut wind = window::Window::default();
-/// let mut but = button::Button::default();
-/// let mut frame = frame::Frame::default();
-/// but.set_callback(move |_| {
-///     let _ = app::handle(CHANGE_FRAME, &wind).unwrap();
-/// });
-/// frame.handle(move |f, ev| {
-///     if ev as i32 == CHANGE_FRAME {
-///         f.set_label("Hello world");
-///         true
-///     } else {
-///         false
-///     }
-/// });
-/// ```
-/// # Errors
-/// Returns Err on error or in use of one of the reserved values.
+/**
+    Send a signal to a window.
+    Integral values from 0 to 30 are reserved.
+    Returns Ok(true) if the event was handled.
+    Returns Ok(false) if the event was not handled.
+    Returns Err on error or in use of one of the reserved values.
+    ```rust,no_run
+    use fltk::{prelude::*, *};
+    const CHANGE_FRAME: i32 = 100;
+    let mut wind = window::Window::default();
+    let mut but = button::Button::default();
+    let mut frame = frame::Frame::default();
+    but.set_callback(move |_| {
+        let _ = app::handle(CHANGE_FRAME, &wind).unwrap();
+    });
+    frame.handle(move |f, ev| {
+        if ev == CHANGE_FRAME.into() {
+            f.set_label("Hello world");
+            true
+        } else {
+            false
+        }
+    });
+    ```
+    # Errors
+    Returns Err on error or in use of one of the reserved values.
+*/
 pub fn handle<I: Into<i32> + Copy + PartialEq + PartialOrd, W: WindowExt>(
     msg: I,
     w: &W,
@@ -1263,30 +1344,32 @@ pub fn handle<I: Into<i32> + Copy + PartialEq + PartialOrd, W: WindowExt>(
     }
 }
 
-/// Send a signal to a window.
-/// Integral values from 0 to 30 are reserved.
-/// Returns Ok(true) if the event was handled.
-/// Returns Ok(false) if the event was not handled.
-/// ```rust,no_run
-/// use fltk::{prelude::*, *};
-/// const CHANGE_FRAME: i32 = 100;
-/// let mut wind = window::Window::default();
-/// let mut but = button::Button::default();
-/// let mut frame = frame::Frame::default();
-/// but.set_callback(move |_| {
-///     let _ = app::handle_main(CHANGE_FRAME).unwrap();
-/// });
-/// frame.handle(move |f, ev| {
-///     if ev as i32 == CHANGE_FRAME {
-///         f.set_label("Hello world");
-///         true
-///     } else {
-///         false
-///     }
-/// });
-/// ```
-/// # Errors
-/// Returns Err on error or in use of one of the reserved values.
+/**
+    Send a signal to the main window.
+    Integral values from 0 to 30 are reserved.
+    Returns Ok(true) if the event was handled.
+    Returns Ok(false) if the event was not handled.
+    ```rust,no_run
+    use fltk::{prelude::*, *};
+    const CHANGE_FRAME: i32 = 100;
+    let mut wind = window::Window::default();
+    let mut but = button::Button::default();
+    let mut frame = frame::Frame::default();
+    but.set_callback(move |_| {
+        let _ = app::handle_main(CHANGE_FRAME).unwrap();
+    });
+    frame.handle(move |f, ev| {
+        if ev == CHANGE_FRAME.into() {
+            f.set_label("Hello world");
+            true
+        } else {
+            false
+        }
+    });
+    ```
+    # Errors
+    Returns Err on error or in use of one of the reserved values.
+*/
 pub fn handle_main<I: Into<i32> + Copy + PartialEq + PartialOrd>(
     msg: I,
 ) -> Result<bool, FltkError> {
@@ -1331,9 +1414,7 @@ pub fn screen_count() -> i32 {
 
 /// Get the screen number based on its coordinates
 pub fn screen_num(x: i32, y: i32) -> i32 {
-    unsafe {
-        fl::Fl_screen_num(x, y)
-    }
+    unsafe { fl::Fl_screen_num(x, y) }
 }
 
 /// Get a screen's dpi resolution
@@ -1385,3 +1466,105 @@ pub unsafe fn open_display() {
 pub unsafe fn close_display() {
     fl::Fl_close_display()
 }
+
+/// Get the clipboard content if it's an image
+pub fn event_clipboard_image() -> Option<crate::image::RgbImage> {
+    unsafe {
+        let image = fl::Fl_event_clipboard();
+        if image.is_null() {
+            None
+        } else {
+            use std::sync::atomic::AtomicUsize;
+            Some(crate::image::RgbImage {
+                inner: image as _,
+                refcount: AtomicUsize::new(1),
+            })
+        }
+    }
+}
+
+/// The event clipboard type
+#[derive(Debug, Clone)]
+pub enum ClipboardEvent {
+    /// Text paste event
+    Text(String),
+    /// image paste event
+    Image(Option<crate::image::RgbImage>),
+}
+
+/// Get the clipboard content type
+pub fn event_clipboard() -> Option<ClipboardEvent> {
+    unsafe {
+        let txt = fl::Fl_event_clipboard_type();
+        let txt = CStr::from_ptr(txt).to_string_lossy().to_string();
+        if txt == "text/plain" {
+            Some(ClipboardEvent::Text(event_text()))
+        } else if txt == "image" {
+            Some(ClipboardEvent::Image(event_clipboard_image()))
+        } else {
+            None
+        }
+    }
+}
+
+#[allow(clippy::missing_safety_doc)]
+/**
+    Send a signal to a window pointer from event_dispatch.
+    Returns true if the event was handled.
+    Returns false if the event was not handled or ignored.
+    ```rust,no_run
+    use fltk::{prelude::*, *};
+    const CHANGE_FRAME: i32 = 100;
+    let mut wind = window::Window::default();
+    let mut but = button::Button::default();
+    let mut frame = frame::Frame::default();
+    wind.end();
+    wind.show();
+    but.set_callback(move |_| {
+        let _ = app::handle_main(CHANGE_FRAME).unwrap();
+    });
+
+    frame.handle(move |f, ev| {
+        if ev == CHANGE_FRAME.into() {
+            f.set_label("Hello world");
+            true
+        } else {
+            false
+        }
+    });
+    unsafe {
+        app::event_dispatch(|ev, winptr| {
+            if ev == CHANGE_FRAME.into() {
+                false // ignore CHANGE_FRAME event
+            } else {
+                app::handle_raw(ev, winptr)
+            }
+        });
+    }
+    ```
+    # Safety
+    The window pointer must be valid
+*/
+pub unsafe fn handle_raw(
+    event: Event,
+    w: WindowPtr,
+) -> bool {
+    fl::Fl_handle_(event.bits(), w as _) != 0
+}
+
+#[allow(clippy::missing_safety_doc)]
+/**
+    The event dispatch function is called after native events are converted to
+    FLTK events, but before they are handled by FLTK. If the dispatch function
+    handler is set, it is up to the dispatch function to call
+    `app::handle2(Event, WindowPtr)` or to ignore the event.
+
+    The dispatch function itself must return false if it ignored the event,
+    or true if it used the event. If you call `app::handle2()`, then
+    this will return the correct value.
+    # Safety
+    The window pointer must not be invalidated
+*/
+pub unsafe fn event_dispatch(f: fn(Event, WindowPtr) -> bool) {
+    fl::Fl_event_dispatch(mem::transmute(f));
+}
