@@ -47,7 +47,7 @@ For faster builds you can enable ninja builds for the C++ source using the "use-
 
 An example hello world application:
 
-```no_run
+```rust,no_run
 use fltk::{app, prelude::*, window::Window};
 let app = app::App::default();
 let mut wind = Window::new(100, 100, 400, 300, "Hello from rust");
@@ -57,7 +57,7 @@ app.run().unwrap();
 ```
 
 Another example showing the basic callback functionality:
-```no_run
+```rust,no_run
 use fltk::{app, button::Button, frame::Frame, prelude::*, window::Window};
 let app = app::App::default();
 let mut wind = Window::new(100, 100, 400, 300, "Hello from rust");
@@ -65,13 +65,13 @@ let mut frame = Frame::new(0, 0, 400, 200, "");
 let mut but = Button::new(160, 210, 80, 40, "Click me!");
 wind.end();
 wind.show();
-but.set_callback(move |_| frame.set_label("Hello World!"));
+but.set_callback(move |_| frame.set_label("Hello World!")); // the closure capture is mutable borrow to our button
 app.run().unwrap();
 ```
 Please check the examples directory for more examples.
 You will notice that all widgets are instantiated with a new() method, taking the x and y coordinates, the width and height of the widget, as well as a label which can be left blank if needed. Another way to initialize a widget is using the builder pattern: (The following buttons are equivalent)
 
-```no_run
+```rust,no_run
 use fltk::{button::*, prelude::*};
 let but1 = Button::new(10, 10, 80, 40, "Button 1");
 
@@ -82,7 +82,7 @@ let but2 = Button::default()
 ```
 
 An example of a counter showing use of the builder pattern:
-```no_run
+```rust,no_run
 use fltk::{app, button::*, frame::*, prelude::*, window::*};
 let app = app::App::default();
 let mut wind = Window::default()
@@ -107,7 +107,7 @@ wind.show();
 /* Event handling */
 ```
 Alternatively, you can use packs to layout your widgets:
-```no_run
+```rust,no_run
 use fltk::{button::*, frame::*, group::*, prelude::*, window::*};
 let mut wind = Window::default().with_size(160, 200).with_label("Counter");
 // Vertical is default. You can choose horizontal using pack.set_type(PackType::Horizontal);
@@ -151,7 +151,12 @@ For custom event handling, the handle() method can be used:
 ```rust,ignore
     some_widget.handle(move |widget, ev: Event| {
         match ev {
-            /* handle ev */
+            Event::Push => {
+                println!("Pushed!");
+                true
+            },
+            /* other events to be handled */
+            _ => false,
         }
     });
 ```
@@ -183,13 +188,13 @@ such as `set_color()`, `set_label_font()`, `set_frame()` etc:
 ## Features
 
 The following are the features offered by the crate:
+- fltk-bundled: Support for bundled versions of cfltk and fltk on selected platforms (requires curl and tar)
+- no-pango: Build without pango support on Linux/BSD.
+- enable-glwindow: Support for drawing using OpenGL functions.
 - use-ninja:  If you have ninja build installed, it builds faster than make or VS
 - system-libpng: Uses the system libpng
 - system-libjpeg: Uses the system libjpeg
 - system-zlib: Uses the system zlib
-- fltk-bundled: Support for bundled versions of cfltk and fltk on selected platforms (requires curl and tar)
-- no-pango: Build without pango support on Linux/BSD.
-- enable-glwindow: Support for drawing using OpenGL functions.
 
 ## Dependencies
 
