@@ -330,6 +330,8 @@ pub unsafe trait WidgetBase: WidgetExt {
     /// * `width` - The width of the widget
     /// * `heigth` - The height of the widget
     /// * `title` - The title or label of the widget
+    /// The title is expected to be a static str or None. 
+    /// To use dynamic strings use `with_label(self, &str)` or `set_label(&mut self, &str)`
     /// labels support special symbols preceded by an `@` [sign](https://www.fltk.org/doc-1.3/symbols.png).
     /// and for the [associated formatting](https://www.fltk.org/doc-1.3/common.html).
     fn new<T: Into<Option<&'static str>>>(
@@ -430,12 +432,16 @@ pub unsafe trait GroupExt: WidgetExt {
         Self: Sized;
     /// Remove a child widget by its index
     fn remove_by_index(&mut self, idx: i32);
-    /// Make the passed widget resizable
+    /// The resizable widget defines both the resizing frame and the resizing behavior of the group and its children.
     fn resizable<W: WidgetExt>(&self, widget: &W)
     where
         Self: Sized;
-    /// Make the window resizable, should be called before `show`
+    /// Make the group itself resizable, should be called before the widget is shown
     fn make_resizable(&mut self, val: bool);
+    /// Adds a widget to the group and makes it the resizable widget
+    fn add_resizable<W: WidgetExt>(&mut self, widget: &W)
+    where
+        Self: Sized;
     /// Clips children outside the group boundaries
     fn set_clip_children(&mut self, flag: bool);
     /// Get whether `clip_children` is set
@@ -454,6 +460,10 @@ pub unsafe trait GroupExt: WidgetExt {
         Self: Sized;
     /// Draw children, the call should be in a `WidgetBase::draw` method
     fn draw_children(&mut self);
+    /// Resets the internal array of widget sizes and positions
+    fn init_sizes(&mut self);
+    /// Get the bounds of all children widgets (left, upper, right, bottom)
+    fn bounds(&self) -> Vec<(i32, i32, i32, i32)>;
 }
 
 /// Defines the methods implemented by all window widgets
@@ -543,7 +553,7 @@ pub unsafe trait InputExt: WidgetExt {
     /// Returns the value inside the input/output widget
     fn value(&self) -> String;
     /// Sets the value inside an input/output widget
-    fn set_value(&self, val: &str);
+    fn set_value(&mut self, val: &str);
     /// Returns the maximum size (in bytes) accepted by an input/output widget
     fn maximum_size(&self) -> i32;
     /// Sets the maximum size (in bytes) accepted by an input/output widget
@@ -1106,8 +1116,6 @@ pub unsafe trait TableExt: GroupExt {
     /// # Errors
     /// Errors on failure to move the cursor
     fn move_cursor(&mut self, r: i32, c: i32) -> Result<(), FltkError>;
-    /// Resets the internal array of widget sizes and positions.
-    fn init_sizes(&mut self);
     /// Returns the scrollbar size
     fn scrollbar_size(&self) -> i32;
     /// Sets the scrollbar size
